@@ -1,5 +1,5 @@
-# WordNet::Extend::Insert.pm version 0.020
-# Updated: 09/29/16
+# WordNet::Extend::Insert.pm version 0.030
+# Updated: 10/13/16
 #                                           
 # Ted Pedersen, University of Minnesota Duluth             
 # tpederse at d.umn.edu
@@ -61,6 +61,7 @@ use WordNet::QueryData;
 use Getopt::Long;
 use File::Spec;
 use File::Copy;
+use File::Find;
 
 our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
@@ -72,7 +73,7 @@ our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.020';
+$VERSION = '0.030';
 
 #**************Variables**********************
 $wn = WordNet::QueryData->new; #to be used to access data from wordnet
@@ -160,6 +161,8 @@ Returns: nothing
 =cut
 sub attach()
 {
+    #need to load in new QueryData
+    $wn = WordNet::QueryData->new;
     my $base = 0; 
     if(scalar @_ == 3)#checks if method entered by object.
     {
@@ -204,6 +207,8 @@ sub attach()
     if($write == 1)
     {
        	my $newOffset = findNewOffset($newSyn[1]);
+	my %CRNotice;
+	my %DataSpace;
 	my %offsetMap; #used to stored changes in offsets.
 	my $indexPos = "";
 	my $dataPos = "";
@@ -273,12 +278,15 @@ sub attach()
 	    {
 		for $tempIn (split("\n"))
 		{
+		    my $spaceTmp = $tempIn;
+		    $spaceTmp =~ /( *)$/;
+		    $DataSpace{$curLine} = length($1);
 		    if($curLine > $wnCRLength)
 		    {
 			my @tempLine = split /\s/, $tempIn;
 			if($changed == 1)
 			{
-			    my $newNewOffset = $tempLine[0] +14;
+			    my $newNewOffset = $tempLine[0] +18;
 			    while(length($newNewOffset) < 8)
 			    {
 				$newNewOffset = "0".$newNewOffset;
@@ -297,15 +305,17 @@ sub attach()
 			    {
 				$newPcnt = "0".$newPcnt;
 			    }
-			    $newOffset = $newOffset + 14; #14 is the length of new data being added.
+			    $newOffset = $newOffset + 18; #18 is the length of new data being added.
 			    $tempIn = "$hypData{'synset_offset'} $hypData{'lex_filenum'} $hypData{'ss_type'} $hypData{'w_cnt'} $hypData{'word_lex_id'} $newPcnt $hypData{'ptr'} ~ $newOffset $pos 0000 | $hypData{'gloss'}";
 			    $changed = 1;
 			}
 		    }
 		    else
 		    {
-			$curLine+=1;
+			$CRNotice{$curLine} = $tempIn;
 		    }
+
+		    $curLine+=1;
 		    
 		    print WNDATATEMP "$tempIn\n";
 		}
@@ -318,6 +328,7 @@ sub attach()
 	    close WNDATATEMP;
 	    open WNDATATEMP, "$dataFile.temp" or die $!;
 
+	    $curLine = 1;
 	    while(<WNDATATEMP>)
 	    {
 		for $tempIn (split("\n"))
@@ -332,13 +343,26 @@ sub attach()
 			
 		    }
 		    $tempIn = join(' ', @tempLine);
-		    		    	    		    	       	    
-		    print WNDATANEW "$tempIn  \n";
+		    		    	    	
+		    if($curLine > $wnCRLength)
+		    {
+			for($i=1; $i <= $DataSpace{$curLine}; $i++)
+			{
+			    $tempIn = $tempIn . " ";
+			}
+			print WNDATANEW "$tempIn\n";
+		    }
+		    else
+		    {
+			print WNDATANEW "$CRNotice{$curLine}\n";
+		    }
+		    $curLine+=1;
 		}
 	    }
 	    print WNDATANEW "$dataPos  \n";
 
 	    $curLine = 1;
+	    $alpha = 1;
 	    while(<WNINDEX>)
 	    {
 		for $tempIn (split("\n"))
@@ -355,8 +379,19 @@ sub attach()
 				$tempIn ="$hypInfo{'lemma'} $hypInfo{'pos'} $hypInfo{'synset_cnt'} $newPcnt $hypInfo{'ptr_symbol'} ~ $hypInfo{'sense_cnt'} $hypInfo{'tagsense_cnt'} $hypInfo{'synset_offset'}";
 			    }
 			}
-			
+
 			my @tempLine = split /\s/, $tempIn;
+			
+			#add in $indexPos alphabetically
+			if($alpha == 1)
+			{
+			    if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			    {
+				print WNINDEXNEW "$indexPos  \n";
+				$alpha = 0;
+			    }
+			}
+			
 			my $tmpPcnt = $tempLine[2];
 			my $offsetPtr = scalar(@tempLine) - 1;
 			while($tmpPcnt > 0)
@@ -375,16 +410,27 @@ sub attach()
 		    {
 			$curLine+=1;
 		    }
-		    print WNINDEXNEW "$tempIn  \n";
+		    print WNINDEXNEW "$tempIn   \n";
 		}
 	    }
-	    print WNINDEXNEW "$indexPos  \n";
 
+	    $alpha = 1;
 	    while(<WNSENSE>)
 	    {
 		for $tempIn (split("\n"))
 		{
+		    
 		    my @tempLine = split /\s/, $tempIn;
+		    #add in $indexSense alphabetically
+		    if($alpha == 1)
+		    {
+			if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			{
+			    print WNSENSENEW "$indexSense\n";
+			    $alpha = 0;
+			}
+		    }
+		    
 		    if(exists $offsetMap{$tempLine[1]})
 		    {
 			$tempLine[1] = "$offsetMap{$tempLine[1]}";
@@ -394,7 +440,7 @@ sub attach()
 		    print WNSENSENEW "$tempIn\n";
 		}
 	    }
-	    print WNSENSENEW "$indexSense\n";
+	    
 	    close WNINDEX;
 	    close WNDATA;
 	    close WNSENSE;
@@ -419,12 +465,15 @@ sub attach()
 	    {
 		for $tempIn (split("\n"))
 		{
+		    my $spaceTmp = $tempIn;
+		    $spaceTmp =~ /( *)$/;
+		    $DataSpace{$curLine} = length($1);
 		    if($curLine > $wnCRLength)
 		    {
 			my @tempLine = split /\s/, $tempIn;	    	    	    
 			if($changed == 1)
 			{
-			    my $newNewOffset = $tempLine[0] +14;
+			    my $newNewOffset = $tempLine[0] +18;
 			    while(length($newNewOffset) < 8)
 			    {
 				$newNewOffset = "0".$newNewOffset;
@@ -443,15 +492,18 @@ sub attach()
 			    {
 				$newPcnt = "0".$newPcnt;
 			    }
-			    $newOffset = $newOffset + 14; #14 is the length of new data being added.
+			    $newOffset = $newOffset + 18; #14 is the length of new data being added.
 			    $tempIn = "$hypData{'synset_offset'} $hypData{'lex_filenum'} $hypData{'ss_type'} $hypData{'w_cnt'} $hypData{'word_lex_id'} $newPcnt $hypData{'ptr'} ~ $newOffset $pos 0000 | $hypData{'gloss'}";
 			    $changed = 1;
 			}
 		    }
 		    else
 		    {
-			$curLine+=1;
+			$CRNotice{$curLine} = $tempIn;
 		    }
+
+		    $curLine+=1;
+		    
 		    print WNDATATEMP "$tempIn\n";
 		}
 	    }
@@ -463,6 +515,7 @@ sub attach()
 	    close WNDATATEMP;
 	    open WNDATATEMP, "$dataFile.temp" or die $!;
 
+	    $curLine = 1;
 	    while(<WNDATATEMP>)
 	    {
 		for $tempIn (split("\n"))
@@ -477,8 +530,20 @@ sub attach()
 			
 		    }
 		    $tempIn = join(' ', @tempLine);
-		    		    	    		    	       	    
-		    print WNDATANEW "$tempIn  \n";
+
+		    if($curLine > $wnCRLength)
+		    {
+			for($i=1; $i <= $DataSpace{$curLine}; $i++)
+			{
+			    $tempIn = $tempIn . " ";
+			}
+			print WNDATANEW "$tempIn\n";
+		    }
+		    else
+		    {
+			print WNDATANEW "$CRNotice{$curLine}\n";
+		    }
+		    $curLine+=1;	    		    	    		    	       	   
 		}
 	    }
 	    print WNDATANEW "$dataPos  \n";
@@ -528,12 +593,23 @@ sub attach()
 		    print WNINDEXNEW "$tempIn  \n";
 		}
 	    }	    
-	    	    	    
+	    	    
+	    $alpha = 1;
 	    while(<WNSENSE>)
 	    {
 		for $tempIn (split("\n"))
 		{
 		    my @tempLine = split /\s/, $tempIn;
+		    #add in $indexSense alphabetically
+		    if($alpha == 1)
+		    {
+			if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			{
+			    print WNSENSENEW "$indexSense\n";
+			    $alpha = 0;
+			}
+		    }
+		    
 		    if(exists $offsetMap{$tempLine[1]})
 		    {
 			$tempLine[1] = "$offsetMap{$tempLine[1]}";
@@ -543,7 +619,7 @@ sub attach()
 		    print WNSENSENEW "$tempIn\n";
 		}
 	    }
-	    print WNSENSENEW "$indexSense\n";
+	    
 	    close WNINDEX;
 	    close WNDATA;
 	    close WNSENSE;
@@ -612,6 +688,8 @@ Returns: nothing
 =cut
 sub merge()
 {
+    #need to load in new QueryData
+    $wn = WordNet::QueryData->new;
     my $base = 0; 
     if(scalar @_ == 3)#checks if method entered by object.
     {
@@ -657,6 +735,8 @@ sub merge()
     {
        	my $newOffset = findNewOffset($newSyn[1]);
 	my %offsetMap; #used to stored changes in offsets.
+	my %CRNotice;
+	my %DataSpace;
 	my $indexPos = "";
 	my $dataPos = "";
 	my $indexSense = "";
@@ -722,11 +802,14 @@ sub merge()
 	    
             my $changed = 0;
 	    my $curLine = 1;
-	    my $newWordLength = length($newSyn[0]) + 1;
+	    my $newWordLength = length($newSyn[0]) + 2;
 	    while(<WNDATA>)
 	    {
 		for $tempIn (split("\n"))
 		{
+		    my $spaceTmp = $tempIn;
+		    $spaceTmp =~ /( *)$/;
+		    $DataSpace{$curLine} = length($1);
 		    if($curLine > $wnCRLength)
 		    {
 			my @tempLine = split /\s/, $tempIn;
@@ -751,8 +834,9 @@ sub merge()
 		    }
 		    else
 		    {
-			$curLine+=1;
+			$CRNotice{$curLine} = $tempIn;			 
 		    }
+		    $curLine+=1;
 
 		    print WNDATATEMP "$tempIn\n";
 		}
@@ -766,6 +850,7 @@ sub merge()
 	    close WNDATATEMP;
 	    open WNDATATEMP, "$dataFile.temp" or die $!;
 
+	    $curLine = 1;
 	    while(<WNDATATEMP>)
 	    {
 		for $tempIn (split("\n"))
@@ -786,10 +871,23 @@ sub merge()
 		    }
 		    $tempIn = join(' ', @tempLine);
 
-		    print WNDATANEW "$tempIn  \n";
+		    if($curLine > $wnCRLength)
+		    {
+			for($i=1; $i <= $DataSpace{$curLine}; $i++)
+			{
+			    $tempIn = $tempIn . " ";
+			}
+			print WNDATANEW "$tempIn\n";
+		    }
+		    else
+		    {
+			print WNDATANEW "$CRNotice{$curLine}\n";
+		    }
+		    $curLine+=1;
 		}
 	    }   
 
+	    my $alpha = 1;
 	    $curLine = 1;
 	    while(<WNINDEX>)
 	    {
@@ -798,6 +896,17 @@ sub merge()
 		    if($curLine > $wnCRLength)
 		    {
 			my @tempLine = split /\s/, $tempIn;
+			
+			#add in $indexPos alphabetically
+			if($alpha == 1)
+			{
+			    if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			    {
+				print WNINDEXNEW "$indexPos  \n";
+				$alpha = 0;
+			    }
+			}
+			
 			my $tmpPcnt = $tempLine[2];
 			my $offsetPtr = scalar(@tempLine) - 1;
 			while($tmpPcnt > 0)
@@ -818,13 +927,23 @@ sub merge()
 		    print WNINDEXNEW "$tempIn  \n";
 		}
 	    }
-	    print WNINDEXNEW "$indexPos  \n";
-	    	   	       
+
+	    $alpha = 1;
 	    while(<WNSENSE>)
 	    {
 		for $tempIn (split("\n"))
 		{
 		    my @tempLine = split /\s/, $tempIn;
+		    #add in $indexSense alphabetically
+		    if($alpha == 1)
+		    {
+			if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			{
+			    print WNSENSENEW "$indexSense\n";
+			    $alpha = 0;
+			}
+		    }
+		    
 		    if(exists $offsetMap{$tempLine[1]})
 		    {
 			$tempLine[1] = "$offsetMap{$tempLine[1]}";
@@ -855,11 +974,14 @@ sub merge()
 	    
             my $changed = 0;
             my $curLine = 1;
-            my $newWordLength = length($newSyn[0]) + 1;
+            my $newWordLength = length($newSyn[0]) + 3;
             while(<WNDATA>)
             {
                 for $tempIn (split("\n"))
                 {
+		    my $spaceTmp = $tempIn;
+		    $spaceTmp =~ /( *)$/;
+		    $DataSpace{$curLine} = length($1);
                     if($curLine > $wnCRLength)
                     {
                         my @tempLine = split /\s/, $tempIn;
@@ -884,8 +1006,9 @@ sub merge()
                     }
                     else
                     {
-                        $curLine+=1;
+			$CRNotice{$curLine} = $tempIn;
                     }
+		    $curLine+=1;
 
                     print WNDATATEMP "$tempIn\n";
                 }
@@ -905,6 +1028,7 @@ sub merge()
 	    close WNDATATEMP;
 	    open WNDATATEMP, "$dataFile.temp" or die $!;
 
+	    $curLine = 1;
 	    while(<WNDATATEMP>)
 	    {
 		for $tempIn (split("\n"))
@@ -925,7 +1049,19 @@ sub merge()
 		    }
 		    $tempIn = join(' ', @tempLine);
 
-		    print WNDATANEW "$tempIn  \n";
+		    if($curLine > $wnCRLength)
+		    {
+			for($i=1; $i <= $DataSpace{$curLine}; $i++)
+			{
+			    $tempIn = $tempIn . " ";
+			}
+			print WNDATANEW "$tempIn\n";
+		    }
+		    else
+		    {
+			print WNDATANEW "$CRNotice{$curLine}\n";
+		    }
+		    $curLine+=1;
 		}
 	    }
 	        	    
@@ -962,11 +1098,22 @@ sub merge()
 		}
 	    }
 
+	    $alpha = 1;
 	    while(<WNSENSE>)
 	    {
 		for $tempIn (split("\n"))
 		{
 		    my @tempLine = split /\s/, $tempIn;
+		    #add in $indexSense alphabetically
+		    if($alpha == 1)
+		    {
+			if(($tempLine[0] cmp $newSyn[0]) == 1 )
+			{
+			    print WNSENSENEW "$indexSense\n";
+			    $alpha = 0;
+			}
+		    }
+		    
 		    if(exists $offsetMap{$tempLine[1]})
 		    {
 			$tempLine[1] = "$offsetMap{$tempLine[1]}";
@@ -1477,11 +1624,49 @@ sub findNewOffset()
     close WNDATA;
     my @data = split /\s/, $dataLastLine;
 
-    $offset = $data[0] + length($dataLastLine) + 3;
+    $offset = $data[0] + length($dataLastLine) + 1;
         
     return $offset;
 }
 
+=item $obj->changeWNLocation()
+
+NOTE: Method not yet implemented, planned for next update.
+
+Allows the user to temporarily choose the location for WordNet
+which can be used to change between different WordNet 
+dictionaries.
+
+Parameters: New location ex. "/usr/local/WordNet-3.0"
+
+Returns: nothing
+
+=cut
+
+#sub changeWNLocation()
+#{
+#    my $base = 0;
+#    if(scalar @_ == 2)
+#    {
+#	$base = 1;#checks if method entered by object.
+#    }
+#    my $newLocation = $_[$base];
+#
+#    #check to see if /dict exists in the new WN location
+#    $newLocation = "/home/csgrads/ruse0008/WordNet-Insert/WordNet-3.0";
+#    
+#    if(-d "$newLocation/dict")
+#    {
+#	$WNHOME = $newLocation;	
+#    }
+#    else
+#    {
+#	my $self = shift;
+#	$self->{error} = 2;
+#	$self->{errorString} = "Desired WordNet location does not contain dict/.";	
+#    }
+#}    
+
 #**************printHelp()**********************
 # Prints indepth help guide to screen.
 #***********************************************
